package UDP::Model::EdgeDir;

use Moose;

has 'alpha' => ( is => 'rw', isa => 'Num', default => 1 );
has 'classes' => ( is => 'rw', isa => 'Int', default => 100 );
#has 'frequency' => ( is => 'rw', isa => 'HashRef', default => sub {my %h = (); return \%h;} );

sub initialize {
    my ($self, $corpus) = @_;
    my %tag_exists;
    foreach my $sentence ($corpus->get_sentences) {
        foreach my $node ($sentence->get_nodes) {
            $tag_exists{$node->tag} = 1;
        }
    }
    $self->classes(scalar keys %tag_exists);
}

sub compute_counts_and_logprob {
    my ($self, $type, $node, $diff_count, $diff_logprob, $count) = @_;
    my $parent = $node->parent;
    my $direction = $node->ord < $parent->ord ? 'L' : 'R';
    my $edge_label = "ch_p_d ".$node->tag." ".$parent->tag." ".$direction;
    my $parent_label = "p_d $direction ".$parent->tag;
    if ($type eq 'del') {
        $diff_count->{$edge_label}--;
        $diff_count->{$parent_label}--;
        $$diff_logprob -= log(($count->{$edge_label} || 0) + ($diff_count->{$edge_label} || 0) + $self->alpha);
        $$diff_logprob += log(($count->{$parent_label} || 0) + ($diff_count->{$parent_label} || 0) + $self->alpha * $self->classes);
    }
    elsif ($type eq 'ins') {
        $$diff_logprob += log(($count->{$edge_label} || 0) + ($diff_count->{$edge_label} || 0) + $self->alpha);
        $$diff_logprob -= log(($count->{$parent_label} || 0) + ($diff_count->{$parent_label} || 0) + $self->alpha * $self->classes);
        $diff_count->{$edge_label}++;
        $diff_count->{$parent_label}++;
    }
}

1;

