package UDP::Model::Subtree;

use Moose;

has 'alpha' => ( is => 'rw', isa => 'Num', default => 2.5 );
has 'ngram_scores' => ( is => 'rw', isa => 'HashRef', default => sub {my %h = (); return \%h;} );
has 'default_score' => ( is => 'rw', isa => 'Num', default => 0.003 );
has 'score_file' => (is => 'rw', isa => 'Str', required => 1 );

sub initialize {
    my ($self, $corpus) = @_;
    my $filename = $self->score_file;
    open (SCORES, "<:utf8", $filename) or die "The file $filename doesn't exist.";
    while (<SCORES>) {
        chomp;
        my ($ngram, $score) = split /\t/;
        $self->ngram_scores->{$ngram} = $score**$self->alpha;
    }
}

sub compute_counts_and_logprob {
    my ($self, $type, $node, $diff_count, $diff_logprob, $count) = @_;
    my $tag_sequence = join " ", map {$_->tag} $node->get_descendants;
#    print STDERR "$$diff_logprob $tag_sequence\n";
    if ($type eq 'del') {
#        print STDERR "D";
        $$diff_logprob -= log($self->ngram_scores->{$tag_sequence} || $self->default_score);
    }
    elsif ($type eq 'ins') {
#        print STDERR "I";
        $$diff_logprob += log($self->ngram_scores->{$tag_sequence} || $self->default_score);
    }
}

=c
sub compute_counts_and_logprob {
    my ($self, $type, $node, $diff_count, $diff_logprob, $count) = @_;
    my $tag_sequence = join " ", map {$_->tag} sort {$a->ord <=> $b->ord} $node->get_unordered_descendants;
#    print STDERR ">>> $$diff_logprob $tag_sequence\n";
    if ($type eq 'del') {
#        print STDERR "D";
        $$diff_logprob -= log($self->ngram_scores->{$tag_sequence} || $self->default_score);
    }
    elsif ($type eq 'ins') {
#        print STDERR "I";
        $$diff_logprob += log($self->ngram_scores->{$tag_sequence} || $self->default_score);
    }
}
=cut

1;

