#!/usr/bin/perl

use strict;
use warnings;
use utf8;
use Getopt::Long;

binmode STDIN, ":utf8";
binmode STDOUT, ":utf8";

my $TAG_COLUMN = 4;
my $FORM_COLUMN = 1;
my $NO_PUNCTUATION = 1;
my $PICK_PUNCTUATION = 0;

GetOptions(
    "tag-column=i"    => \$TAG_COLUMN,
    "form-column=i"   => \$FORM_COLUMN,
    "no-punctuation=i"=> \$NO_PUNCTUATION,
    "pick-punctuation=i"=> \$PICK_PUNCTUATION,
);

my %count;
my %ngram_count;
my %ngram_count_total;
my %sent2tags;
my %examples;
my $total_count = 0;
my $total_count_total = 0;

my @words;
my @tags;

my $sent_counter = 0;

my %is_punc;

while (<>) {
    chomp;
    my $line = $_;
    if ($line =~ /^\d/) {
        my @items = split /\t/, $line;
        push @words, lc($items[$FORM_COLUMN]);
        push @tags, $items[$TAG_COLUMN];
        $is_punc{$items[$TAG_COLUMN]} = 1 if $items[5] eq '.';
        $examples{$items[$TAG_COLUMN]} = "" if !defined $examples{$items[$TAG_COLUMN]};
        $examples{$items[$TAG_COLUMN]} .= "$items[$FORM_COLUMN] " if length $examples{$items[$TAG_COLUMN]} < 100;
        $total_count_total++;
    }
    else {
        $sent_counter++;
        print STDERR "$sent_counter " if $sent_counter % 10000 == 0;
        my $w = join " ", @words;
        my $t = join " ", @tags;
        if (@words < 30) {
            $count{$w}++;
            $sent2tags{$w} = $t;
        }
        @words = ();
        @tags = ();
    }
}

print STDERR "The corpus has been read.\n";

foreach my $sent (keys %count) {
    my @words = split /\s/, $sent;
    my @tags = split /\s/, $sent2tags{$sent};
    next if @words < 10;
    # unigrams
    foreach my $i (0 .. $#words) {
        my $preceding = $tags[$i-1] || '<s>';
        my $following = $tags[$i+1] || '<s>';
        my $ngram = "$preceding | $tags[$i] | $following";
        my $ngram2 = "$tags[$i]";
        $ngram_count_total{$ngram2}++;
        my $new_sent = join (" ", map {$words[$_]} grep {$_ != $i} (0 .. $#words));
        if ($count{$new_sent}) {
            $ngram_count{$ngram2}++;
            $total_count++;
        }
    }

    # bigrams
    foreach my $i (0 .. $#words-1) {
        my $preceding = $tags[$i-1] || '<s>';
        my $following = $tags[$i+2] || '<s>';
        my $ngram = "$preceding | $tags[$i] $tags[$i+1] | $following";
        my $ngram2 = "$tags[$i] $tags[$i+1]";
        $ngram_count_total{$ngram2}++;
        my $new_sent = join (" ", map {$words[$_]} grep {$_ != $i && $_ != $i+1} (0 .. $#words));
        if ($count{$new_sent}) {
            $ngram_count{$ngram2}++;
            $total_count++;
        }
    }

    # trigrams
    foreach my $i (0 .. $#words-2) {
        my $preceding = $tags[$i-1] || '<s>';
        my $following = $tags[$i+3] || '<s>';
        my $ngram = "$preceding | $tags[$i] $tags[$i+1] $tags[$i+2] | $following";
        my $ngram2 = "$tags[$i] $tags[$i+1] $tags[$i+2]";
        $ngram_count_total{$ngram2}++;
        my $new_sent = join (" ", map {$words[$_]} grep {$_ != $i && $_ != $i+1 && $_ != $i+2} (0 .. $#words));
        if ($count{$new_sent}) {
            $ngram_count{$ngram2}++;
            $total_count++;
        }
    }

#    # tetragrams and longer
#    foreach my $n (3 .. 9) {
#        foreach my $i (0 .. $#words - $n) {
#            my $ngram = join (" ", @tags[$i .. ($i+$n)]);
#            $ngram_count_total{$ngram}++;
#            $total_count_total++;
#            my $new_sent = join (" ", map {$words[$_]} ((0 .. ($i - 1)), (($i + $n + 1) .. $#words)));
#            if ($count{$new_sent}) {
#                $ngram_count{$ngram}++;
#                $total_count++;
#            }
#        }
#    }

}

my $alpha = $total_count / $total_count_total;
my $beta = 1;
print STDERR "alpha = $alpha, beta = $beta\n";

foreach my $t (keys %ngram_count_total) {
#    $tag_count{$t} = (($tag_count{$t} || 0) + 0.1) / ($tag_count_total{$t} + 0.1);
#    $ngram_count{$t} = $ngram_count{$t} * $total_count_total / $ngram_count_total{$t} / $total_count;
#    $ngram_count{$t} = (($ngram_count{$t} || 0) + 1) * $total_count_total / ($ngram_count_total{$t}) / ($total_count + (scalar keys %ngram_count_total));
    $ngram_count{$t} = (($ngram_count{$t} || 0) + $alpha) * ($total_count_total + $beta*(scalar keys %ngram_count_total)) / ($ngram_count_total{$t} + $beta) / ($total_count + $alpha*(scalar keys %ngram_count_total));
    if ($PICK_PUNCTUATION && $is_punc{$t}) {
        $ngram_count{$t} = 100;
    }
}

my @ordered = sort {$ngram_count{$a} <=> $ngram_count{$b}} keys %ngram_count;

foreach my $t (@ordered) {
#    $tag_count{$t} *= 0.9 / $tag_count{$ordered[$#ordered]};
    my $prob = 1 - 4**(-$ngram_count{$t});
    print "$t\t$ngram_count{$t}\t$prob\t$ngram_count_total{$t}\n";
}

